package metrics_test

import (
	cfg "github.com/pivotal-cf-experimental/mysql-metrics/config"
	. "github.com/pivotal-cf-experimental/mysql-metrics/metrics"

	"fmt"
	"io/ioutil"
	"os"
	"path/filepath"
	"strings"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
)

var _ = Describe("MetricMappingConfig", func() {

	var (
		configFilepath string
		tempDir        string
		metricName     string
		key            string
		unit           string

		metricName2 string
		key2        string
		unit2       string

		internalMetricName string
		internalKey        string

		leaderFollowerMetricName string
		leaderFollowerKey        string
		leaderFollowerUnit       string
	)

	Describe("with a fully formed, correct yaml file", func() {
		BeforeEach(func() {
			metricName = "dummyMetric"
			key = "/p-mysql/performance/dummyMetric"
			unit = "jiff"

			metricName2 = "dummyMetric2"
			key2 = "/p-mysql/performance/dummyMetric2"
			unit2 = "jiff2"

			internalMetricName = "available"
			internalKey = "/p-mysql/available"

			leaderFollowerMetricName = "is_follower"
			leaderFollowerKey = "/p-mysql/follower/is_follower"
			leaderFollowerUnit = "number"

			tempDir, err := ioutil.TempDir("", "")
			Expect(err).NotTo(HaveOccurred())

			configFilepath = filepath.Join(tempDir, "metric-mapping-config.yml")

			yaml := `
				---
				mysql_metrics:
				  %s:
				    key: %s
				    unit: %s
				  %s:
				    key: %s
				    unit: %s
				internal_metrics:
				  %s:
				    key: %s
				leader_follower_metrics:
				  %s:
				    key: %s
				    unit: %s
			`
			yaml = strings.Replace(yaml, "\t", "", -1)
			configString := fmt.Sprintf(yaml, metricName, key, unit,
				metricName2, key2, unit2,
				internalMetricName, internalKey,
				leaderFollowerMetricName, leaderFollowerKey, leaderFollowerUnit)

			err = ioutil.WriteFile(configFilepath, []byte(configString), os.ModePerm)
			Expect(err).NotTo(HaveOccurred())
		})

		AfterEach(func() {
			err := os.RemoveAll(tempDir)
			Expect(err).NotTo(HaveOccurred())
		})

		It("reads the config file", func() {
			var err error
			config := &MetricMappingConfig{}
			err = cfg.LoadFromFile(configFilepath, config)

			Expect(err).NotTo(HaveOccurred())
			Expect(config).NotTo(BeNil())
			Expect(config.MysqlMetricMappings).NotTo(BeNil())
			Expect(len(config.MysqlMetricMappings)).To(Equal(2))
			Expect(config.InternalMetricMappings).NotTo(BeNil())
			Expect(len(config.InternalMetricMappings)).To(Equal(1))
			Expect(config.LeaderFollowerMetricMappings).NotTo(BeNil())
			Expect(len(config.LeaderFollowerMetricMappings)).To(Equal(1))

			mappings := config.GetMysqlMetricMappings()
			mapping := mappings[metricName]
			Expect(mapping.Key).To(Equal(key))
			Expect(mapping.Unit).To(Equal(unit))
			mapping = mappings[metricName2]
			Expect(mapping.Key).To(Equal(key2))
			Expect(mapping.Unit).To(Equal(unit2))

			internalMappings := config.GetInternalMetricMappings()
			internalMapping := internalMappings[internalMetricName]
			Expect(internalMapping.Key).To(Equal(internalKey))

			leaderFollowerMappings := config.GetLeaderFollowerMetricMappings()
			leaderFollowerMapping := leaderFollowerMappings[leaderFollowerMetricName]
			Expect(leaderFollowerMapping.Key).To(Equal(leaderFollowerKey))
			Expect(leaderFollowerMapping.Unit).To(Equal(leaderFollowerUnit))
		})
	})
})
